@@ -14,29 +14,27 @@ use warnings;
 our @ISA = qw(Exporter);
 our @EXPORT_OK = qw(debug $DEBUG carp_once);
 our %EXPORT_TAGS = (all => [ @EXPORT_OK ]);
+our $VERSION = '0.52';
 
-our ($DEBUG, $DEBUG_fh, %_CARPED, 
+our ($DEBUG, $DEBUG_FH, %_CARPED, 
      $OLD_SIG__WARN__, $OLD_SIG__DIE__, $OLD_CARP_VERBOSE);
-our $VERSION = $Forks::Super::Util::VERSION;
 
+## no critic (BriefOpen,TwoArgOpen)
 (uc($ENV{FORKS_SUPER_DEBUG} || "") eq 'TTY'
- && open($DEBUG_fh, '>', $^O eq 'MSWin32' ? 'CON' : '/dev/tty'))
-  or open($DEBUG_fh, '>&2')
-  or $DEBUG_fh = *STDERR
+ && open($DEBUG_FH, '>', $^O eq 'MSWin32' ? 'CON' : '/dev/tty'))
+  or open($DEBUG_FH, '>&2')
+  or $DEBUG_FH = *STDERR
   or carp_once("Forks::Super: Debugging not available in module!\n");
-$DEBUG_fh->autoflush(1);
+## use critic
+$DEBUG_FH->autoflush(1);
 $DEBUG = !!$ENV{FORKS_SUPER_DEBUG} || '0';
 
-# open filehandle to tty for emergency debugging as we may clobber STDERR
-open ::__XXXXXX__, '>>', $^O eq 'MSWin32' ? "CON" : "/dev/tty";
-(*::__XXXXXX__)->autoflush(1);
-
 sub init {
 }
 
 sub debug {
   my @msg = @_;
-  print $DEBUG_fh $$," ",Forks::Super::Util::Ctime()," ",@msg,"\n";
+  print {$DEBUG_FH} $$," ",Forks::Super::Util::Ctime()," ",@msg,"\n";
   return;
 }
 
@@ -50,30 +48,29 @@ sub carp_once {
     shift @msg;
   }
   return if $_CARPED{"$p:$f:$l:$z"}++;
-  carp @msg;
+  return carp @msg;
 }
 
 # load or emulate Carp::Always for the remainder of the program
-sub _use_Carp_Always {
-  _emulate_Carp_Always();
-}
-
-sub _emulate_Carp_Always {
+sub use_Carp_Always {
   $OLD_CARP_VERBOSE = $Carp::Verbose if !defined($OLD_CARP_VERBOSE);
   $Carp::Verbose = 'verbose';
   if (!defined($OLD_SIG__WARN__)) {
     $OLD_SIG__WARN__ = $SIG{__WARN__} || 'DEFAULT';
     $OLD_SIG__DIE__ = $SIG{__DIE__} || 'DEFAULT';
   }
+  ## no critic (RequireCarping)
   $SIG{__WARN__} = sub { warn &Carp::longmess };
   $SIG{__DIE__} = sub { warn &Carp::longmess };
-  1;
+  return 1;
 }
 
-sub _deemulate_Carp_Always {
+# stop emulation of Carp::Always
+sub no_Carp_Always {
   $Carp::Verbose = $OLD_CARP_VERBOSE || 0;
   $SIG{__WARN__} = $OLD_SIG__WARN__ || 'DEFAULT';
   $SIG{__DIE__} = $OLD_SIG__DIE__ || 'DEFAULT';
+  return;
 }
 
 1;
