@@ -17,7 +17,7 @@ use warnings;
 our @ISA = qw(Exporter);
 our @EXPORT_OK = qw(queue_job);
 our %EXPORT_TAGS = (all => \@EXPORT_OK);
-our $VERSION = $Forks::Super::Util::VERSION;
+our $VERSION = '0.52';
 
 our (@QUEUE, $QUEUE_MONITOR_PID, $QUEUE_MONITOR_PPID, $QUEUE_MONITOR_FREQ);
 our $QUEUE_DEBUG = $ENV{FORKS_SUPER_QUEUE_DEBUG} || 0;
@@ -31,9 +31,9 @@ our ($MAIN_PID,$_LOCK) = ($$,0);
 # if a child process finishes while the  run_queue()  function is running,
 # we will usually have to restart that function in order to make sure
 # that jobs are dispatched quickly and in the correct order. The SIGCHLD
-# handler sets the flag  $_REAP , and if we check that flag  run_queue()
+# handler sets a  $REAP  flag, and if we check that flag  run_queue()
 # will do its job properly.
-our ($CHECK_FOR_REAP, $_REAP) = (1,0);
+our $CHECK_FOR_REAP = 1;
 
 # use var $Forks::Super::QUEUE_INTERRUPT, not lexical package var
 
@@ -56,6 +56,7 @@ sub init {
   if (grep {/USR1/} keys %SIG) {
     $Forks::Super::QUEUE_INTERRUPT = 'USR1';
   }
+  return;
 }
 
 sub init_child {
@@ -65,6 +66,7 @@ sub init_child {
       && $Forks::Super::SysInfo::CONFIG{SIGUSR1}) {
     $SIG{$Forks::Super::QUEUE_INTERRUPT} = 'DEFAULT';
   }
+  return;
 }
 
 #
@@ -96,12 +98,14 @@ sub _launch_queue_monitor {
   } else {
     _launch_queue_monitor_fork();
   }
+  return;
 }
 
 sub _check_queue {
-  # XXX - check_queue call triggered by a SIGALRM. 
-  #       do we want to log it or do any other special handling?
+  # check_queue call triggered by a SIGALRM. 
+  # XXX - do we want to log it or do any other special handling?
   check_queue();
+  return;
 }
 
 sub _launch_queue_monitor_setitimer {
@@ -112,6 +116,7 @@ sub _launch_queue_monitor_setitimer {
 
   Time::HiRes::setitimer(
 	&Time::HiRes::ITIMER_REAL, $QUEUE_MONITOR_FREQ, $QUEUE_MONITOR_FREQ);
+  return;
 }
 
 sub _launch_queue_monitor_fork {
@@ -210,11 +215,13 @@ sub _kill_queue_monitor {
       }
     }
   }
+  return;
 }
 
 
 sub _cleanup {
   _kill_queue_monitor();
+  return;
 }
 
 #
@@ -248,12 +255,13 @@ sub queue_job {
 }
 
 sub _check_for_reap {
-  if ($CHECK_FOR_REAP && $_REAP > 0) {
+  if ($CHECK_FOR_REAP && $Forks::Super::Sigchld::REAP > 0) {
     if ($DEBUG || $QUEUE_DEBUG) {
       debug("reap during queue examination -- restart");
     }
     return 1;
   }
+  return;
 }
 
 #
@@ -289,64 +297,64 @@ sub run_queue {
   #   go through the list and attempt to launch each job in order.
 
   debug('run_queue(): examining deferred jobs') if $DEBUG || $QUEUE_DEBUG;
-  my $job_was_launched;
-  do {
-    $job_was_launched = 0;
-    $_REAP = 0;
-    my @deferred_jobs = grep {
-      defined($_->{state}) && $_->{state} eq 'DEFERRED'
-    } @Forks::Super::ALL_JOBS;
-    @deferred_jobs = sort {
-      ($b->{queue_priority} || 0) 
-	      <=> 
-      ($a->{queue_priority} || 0)
-    } @deferred_jobs;
-
-    foreach my $job (@deferred_jobs) {
-      if ($job->can_launch) {
-	if ($job->{debug}) {
-	  debug("Launching deferred job $job->{pid}")
-	}
-	$job->{state} = 'LAUNCHING';
-
-	# if this loop gets interrupted to handle a child,
-	# we might be launching jobs in the wrong order.
-	# If we detect that an interruption has happened,
-	# abort and restart the loop.
-	#
-	# To disable this check, set 
-	# $Forks::Super::Queue::CHECK_FOR_REAP = 0
-
-	if (_check_for_reap()) {
-	  $job->{state} = 'DEFERRED';
-	  $job_was_launched = 1;
-	  last;
-	}
-	my $pid = $job->launch();
-	if ($pid == 0) {
-	  if (defined($job->{sub}) || defined($job->{cmd})
-	      || defined($job->{exec})) {
-	    $_LOCK--;
-	    croak "Forks::Super::run_queue(): ",
-	      "fork on deferred job unexpectedly returned ",
-		"a process id of 0!\n";
-	  }
-	  $_LOCK--;
-	  croak "Forks::Super::run_queue(): ",
-	    "deferred job must have a 'sub', 'cmd', or 'exec' option!\n";
-	}
-	$job_was_launched = 1;
-	last;
-      } elsif ($job->{debug}) {
-	debug("Still must wait to launch job ", $job->toShortString());
-      }
-    }
-  } while ($job_was_launched);
-
+  1 while _attempt_to_launch_deferred_jobs();
   $_LOCK--;
   return;
 }
 
+sub _get_deferred_jobs {
+    my @deferred_jobs = grep { 
+	defined($_->{state}) && $_->{state} eq 'DEFERRED' 
+    } @Forks::Super::ALL_JOBS;
+    @deferred_jobs = sort { 
+	($b->{queue_priority}||0) <=> ($a->{queue_priority}||0)
+    } @deferred_jobs;
+    return @deferred_jobs;
+}
+
+sub _attempt_to_launch_deferred_jobs {
+    $Forks::Super::Sigchld::REAP = 0;
+    foreach my $job (_get_deferred_jobs()) {
+	if ($job->can_launch) {
+	    if ($job->{debug}) {
+		debug("Launching deferred job $job->{pid}")
+	    }
+	    $job->{state} = 'LAUNCHING';
+
+	    # if this loop gets interrupted to handle a child,
+	    # we might be launching jobs in the wrong order.
+	    # If we detect that an interruption has happened,
+	    # abort and restart the loop.
+	    #
+	    # To disable this check, set 
+	    # $Forks::Super::Queue::CHECK_FOR_REAP = 0
+
+	    if (_check_for_reap()) {
+		$job->{state} = 'DEFERRED';
+		return 1;
+	    }
+	    my $pid = $job->launch();
+	    if ($pid == 0) {
+		if (defined($job->{sub}) || defined($job->{cmd})
+		    || defined($job->{exec})) {
+		    $_LOCK--;
+		    croak "Forks::Super::run_queue(): ",
+		        "fork on deferred job unexpectedly returned ",
+		        "a process id of 0!\n";
+		}
+		$_LOCK--;
+		croak "Forks::Super::run_queue(): ",
+		    "deferred job must have a ",
+		    "'sub', 'cmd', or 'exec' option!\n";
+	    }
+	    return 1;
+	} elsif ($job->{debug}) {
+	    debug("Still must wait to launch job ", $job->toShortString());
+	}
+    }        # next deferred job
+    return 0;
+}
+
 sub suspend_resume_jobs {
   my @jobs = grep {
     defined($_->{suspend}) &&
@@ -363,19 +371,16 @@ sub suspend_resume_jobs {
 
   foreach my $job (@jobs) {
     no strict 'refs';
+    my $job_is_suspended = $job->{state} =~ /SUSPEND/;
     my $action = $job->{suspend}->();
-    if ($action > 0) {
-      if ($job->{state} =~ /SUSPEND/) {
-	debug("Forks::Super::Queue: suspend callback value $action for ",
-	      "job ", $job->{pid}, " ... resuming") if $job->{debug};
-	$job->resume;
-      }
-    } elsif ($action < 0) {
-      if ($job->{state} eq 'ACTIVE') {
+    if ($action < 0 && ! $job_is_suspended) {
+	$job->suspend;
 	debug("Forks::Super::Queue: suspend callback value $action for ",
 	      "job ", $job->{pid}, " ... suspending") if $job->{debug};
-	$job->suspend;
-      }
+    } elsif ($action > 0 && $job_is_suspended) {
+	$job->resume;
+	debug("Forks::Super::Queue: suspend callback value $action for ",
+	      "job ", $job->{pid}, " ... resuming") if $job->{debug};
     }
   }
 
@@ -437,6 +442,7 @@ sub Forks::Super::Queue::QueueMonitorFreq::STORE {
   _kill_queue_monitor();
   check_queue();
   _launch_queue_monitor() if @QUEUE > 0;
+  return;
 }
 
 #############################################################################
